@@ -1,22 +1,8 @@
-/***************************************************************************
- *   This file is part of KDevelop                                         *
- *   Copyright 2008 Niko Sams <niko.sams@gmail.com>                        *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU Library General Public License as       *
- *   published by the Free Software Foundation; either version 2 of the    *
- *   License, or (at your option) any later version.                       *
- *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *   GNU General Public License for more details.                          *
- *                                                                         *
- *   You should have received a copy of the GNU Library General Public     *
- *   License along with this program; if not, write to the                 *
- *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
- ***************************************************************************/
+/*
+    SPDX-FileCopyrightText: 2008 Niko Sams <niko.sams@gmail.com>
+
+    SPDX-License-Identifier: LGPL-2.0-or-later
+*/
 
 #include "typebuilder.h"
 
@@ -26,8 +12,8 @@
 #include <language/duchain/ducontext.h>
 #include <language/duchain/declaration.h>
 #include <language/duchain/types/integraltype.h>
+#include <language/duchain/types/arraytype.h>
 #include "../declarations/classdeclaration.h"
-#include "../types/indexedcontainer.h"
 #include "../types/integraltypeextended.h"
 #include "../types/structuretype.h"
 #include <duchaindebug.h>
@@ -57,8 +43,36 @@ TypeBuilder::~TypeBuilder()
 
 AbstractType::Ptr TypeBuilder::parseType(QString type, AstNode* node)
 {
-    uint iType = 0;
     type = type.trimmed();
+
+    if (type.contains('|')) {
+        QList<AbstractType::Ptr> types;
+        foreach (const QString& t, type.split('|')) {
+            AbstractType::Ptr subType = parseType(t, node);
+            if (!(subType.dynamicCast<IntegralType>() && subType.staticCast<IntegralType>()->dataType() == IntegralType::TypeMixed)) {
+                types << parseType(t, node);
+            }
+        }
+        UnsureType::Ptr ret(new UnsureType());
+        foreach (const AbstractType::Ptr& t, types) {
+            ret->addType(t->indexed());
+        }
+        return ret;
+    }
+
+    if (type.endsWith(QLatin1String("[]"))) {
+        KDevelop::ArrayType* a_type = new KDevelop::ArrayType();
+        a_type->setElementType(parseSimpleType(type.left(type.length() - 2), node));
+        return AbstractType::Ptr(a_type);
+    } else {
+        return parseSimpleType(type, node);
+    }
+}
+
+AbstractType::Ptr TypeBuilder::parseSimpleType(QString type, AstNode* node)
+{
+    Q_UNUSED(node)
+    uint iType = 0;
     if (!type.compare(QLatin1String("int"), Qt::CaseInsensitive) || !type.compare(QLatin1String("integer"), Qt::CaseInsensitive)) {
         iType = IntegralType::TypeInt;
     } else if (!type.compare(QLatin1String("float"), Qt::CaseInsensitive) || !type.compare(QLatin1String("double"), Qt::CaseInsensitive)) {
@@ -101,23 +115,6 @@ AbstractType::Ptr TypeBuilder::parseType(QString type, AstNode* node)
         if (decl && decl->abstractType()) {
             return decl->abstractType();
         }
-        if (type.contains('|')) {
-            QList<AbstractType::Ptr> types;
-            foreach (const QString& t, type.split('|')) {
-                AbstractType::Ptr subType = parseType(t, node);
-                if (!(IntegralType::Ptr::dynamicCast(subType) && IntegralType::Ptr::staticCast(subType)->dataType() == IntegralType::TypeMixed)) {
-                    types << parseType(t, node);
-                }
-            }
-            if (!type.isEmpty()) {
-                UnsureType::Ptr ret(new UnsureType());
-                foreach (const AbstractType::Ptr& t, types) {
-                    ret->addType(t->indexed());
-                }
-                //qCDebug(DUCHAIN) << type << ret->toString();
-                return AbstractType::Ptr::staticCast(ret);
-            }
-        }
         iType = IntegralType::TypeMixed;
     }
     AbstractType::Ptr ret(new IntegralType(iType));
@@ -322,6 +319,20 @@ void TypeBuilder::visitClassStatement(ClassStatementAst *node)
     } else if (node->constsSequence) {
         //class constant
         TypeBuilderBase::visitClassStatement(node);
+    } else if (node->propertyType) {
+        m_gotTypeFromTypeHint = true;
+        AbstractType::Ptr phpDocTypehint = parseDocComment(node, QStringLiteral("var"));
+        AbstractType::Ptr type = propertyType(node, phpDocTypehint, editor(), currentContext());
+
+        injectType(type);
+        TypeBuilderBase::visitClassStatement(node);
+        clearLastType();
+        m_gotTypeFromTypeHint = false;
+
+        if (m_gotTypeFromDocComment) {
+            clearLastType();
+            m_gotTypeFromDocComment = false;
+        }
     } else {
         //member-variable
         parseDocComment(node, QStringLiteral("var"));
@@ -335,7 +346,7 @@ void TypeBuilder::visitClassStatement(ClassStatementAst *node)
 
 void TypeBuilder::visitClassVariable(ClassVariableAst *node)
 {
-    if (!m_gotTypeFromDocComment) {
+    if (!m_gotTypeFromDocComment && !m_gotTypeFromTypeHint) {
         if (node->value) {
             openAbstractType(getTypeForNode(node->value));
         } else {
@@ -479,30 +490,30 @@ void TypeBuilder::visitStatement(StatementAst* node)
         AbstractType::Ptr type = getTypeForNode(node->returnExpr);
         if (type) {
             // ignore references for return values, PHP does so as well
-            if ( ReferenceType::Ptr rType = ReferenceType::Ptr::dynamicCast(type) ) {
+            if ( auto rType = type.dynamicCast<ReferenceType>() ) {
                 type = rType->baseType();
             }
             if (ft->returnType() && !ft->returnType()->equals(type.data())) {
-                bool existingTypeIsCallable = ft->returnType().cast<IntegralTypeExtended>() &&
-                    ft->returnType().cast<IntegralTypeExtended>()->dataType() == IntegralTypeExtended::TypeCallable;
-                bool newTypeIsCallable = type.cast<IntegralTypeExtended>() &&
-                    type.cast<IntegralTypeExtended>()->dataType() == IntegralTypeExtended::TypeCallable;
-                if (ft->returnType().cast<IntegralType>()
-                    && ft->returnType().cast<IntegralType>()->dataType() == IntegralType::TypeMixed)
+                bool existingTypeIsCallable = ft->returnType().dynamicCast<IntegralTypeExtended>() &&
+                    ft->returnType().staticCast<IntegralTypeExtended>()->dataType() == IntegralTypeExtended::TypeCallable;
+                bool newTypeIsCallable = type.dynamicCast<IntegralTypeExtended>() &&
+                    type.staticCast<IntegralTypeExtended>()->dataType() == IntegralTypeExtended::TypeCallable;
+                if (ft->returnType().dynamicCast<IntegralType>()
+                    && ft->returnType().staticCast<IntegralType>()->dataType() == IntegralType::TypeMixed)
                 {
                     //don't add TypeMixed to the list, just ignore
                     ft->setReturnType(type);
-                } else if ((existingTypeIsCallable && type.cast<FunctionType>()) || (newTypeIsCallable && ft->returnType().cast<FunctionType>())) {
+                } else if ((existingTypeIsCallable && type.dynamicCast<FunctionType>()) || (newTypeIsCallable && ft->returnType().dynamicCast<FunctionType>())) {
                     //If one type is "callable" and the other a real function, the result is just a "callable".
                     ft->setReturnType(AbstractType::Ptr(new IntegralTypeExtended(IntegralTypeExtended::TypeCallable)));
                 } else {
                     UnsureType::Ptr retT;
-                    if (ft->returnType().cast<UnsureType>()) {
+                    if (ft->returnType().dynamicCast<UnsureType>()) {
                         //qCDebug(DUCHAIN) << "we already have an unsure type";
-                        retT = ft->returnType().cast<UnsureType>();
-                        if (type.cast<UnsureType>()) {
+                        retT = ft->returnType().staticCast<UnsureType>();
+                        if (type.dynamicCast<UnsureType>()) {
                             //qCDebug(DUCHAIN) << "add multiple to returnType";
-                            FOREACH_FUNCTION(const IndexedType& t, type.cast<UnsureType>()->types) {
+                            FOREACH_FUNCTION(const IndexedType& t, type.staticCast<UnsureType>()->types) {
                                 retT->addType(t);
                             }
                         } else {
@@ -510,15 +521,15 @@ void TypeBuilder::visitStatement(StatementAst* node)
                             retT->addType(type->indexed());
                         }
                     } else {
-                        if (type.cast<UnsureType>()) {
-                            retT = type.cast<UnsureType>();
+                        if (type.dynamicCast<UnsureType>()) {
+                            retT = type.staticCast<UnsureType>();
                         } else {
                             retT = new UnsureType();
                             retT->addType(type->indexed());
                         }
                         retT->addType(ft->returnType()->indexed());
                     }
-                    ft->setReturnType(AbstractType::Ptr::staticCast(retT));
+                    ft->setReturnType(retT);
                 }
             } else {
                 ft->setReturnType(type);
@@ -541,12 +552,12 @@ void TypeBuilder::visitStatement(StatementAst* node)
         v.visitNode(foreachNode);
         DUChainReadLocker lock(DUChain::lock());
         bool foundType = false;
-        if (StructureType::Ptr type = StructureType::Ptr::dynamicCast(v.result().type())) {
+        if (auto type = v.result().type().dynamicCast<StructureType>()) {
             ClassDeclaration *classDec = dynamic_cast<ClassDeclaration*>(type->declaration(currentContext()->topContext()));
             if (!classDec) {
                 ///FIXME: this is just a hack for https://bugs.kde.org/show_bug.cgi?id=269369
                 ///       a proper fix needs full fledged two-pass, i.e. get rid of PreDeclarationBuilder
-                // 0 == global lookup and the delcaration is found again...
+                // 0 == global lookup and the declaration is found again...
                 classDec = dynamic_cast<ClassDeclaration*>(type->declaration(nullptr));
             }
             if (classDec) {
@@ -572,7 +583,11 @@ void TypeBuilder::visitStatement(StatementAst* node)
                     }
                 }
             }
+        } else if ( auto a_type = v.result().type().dynamicCast<ArrayType>() ) {
+            injectType(a_type->elementType());
+            foundType = true;
         }
+
         if (!foundType) {
             injectType(AbstractType::Ptr(new IntegralType(IntegralType::TypeMixed)));
         }
@@ -582,19 +597,46 @@ void TypeBuilder::visitStatement(StatementAst* node)
 void TypeBuilder::visitCatchItem(Php::CatchItemAst *node)
 {
     TypeBuilderBase::visitCatchItem(node);
-    DeclarationPointer dec = findDeclarationImport(ClassDeclarationType,
-                                                   identifierForNamespace(node->catchClass, m_editor));
-    if (dec && dec->abstractType()) {
-        openAbstractType(dec->abstractType());
-        closeType();
-    }
 
+    const KDevPG::ListNode< NamespacedIdentifierAst* >* it = node->catchClassSequence->front();
+
+    if (node->catchClassSequence->count() == 1) {
+        DeclarationPointer dec = findDeclarationImport(ClassDeclarationType,
+                                                       identifierForNamespace(it->element, m_editor));
+
+        if (dec && dec->abstractType()) {
+            openAbstractType(dec->abstractType());
+            closeType();
+        }
+    } else {
+        UnsureType::Ptr decs(new UnsureType());
+
+        forever {
+            DeclarationPointer dec = findDeclarationImport(ClassDeclarationType,
+                                                           identifierForNamespace(it->element, m_editor));
+
+            if (dec && dec->abstractType()) {
+                decs->addType(dec->abstractType()->indexed());
+            }
+
+            if ( it->hasNext() ) {
+                it = it->next;
+            } else {
+                break;
+            }
+        }
+
+        if (decs) {
+            openAbstractType(decs);
+            closeType();
+        }
+    }
 }
 
 void TypeBuilder::visitVarExpression(Php::VarExpressionAst *node)
 {
     if (hasCurrentContextType() && node->isGenerator != -1 && !m_gotReturnTypeFromDocComment) {
-        FunctionType::Ptr ft = FunctionType::Ptr::dynamicCast(currentContextType());
+        auto ft = currentContextType().dynamicCast<FunctionType>();
         static QualifiedIdentifier generatorQId(QStringLiteral("generator"));
         generatorQId.setExplicitlyGlobal(true);
         DeclarationPointer generatorDecl = findDeclarationImport(ClassDeclarationType, generatorQId);
